package org.obm.push.backend.obm22.calendar;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.obm.push.backend.BackendSession;
import org.obm.push.backend.DataDelta;
import org.obm.push.backend.FolderType;
import org.obm.push.backend.IApplicationData;
import org.obm.push.backend.ItemChange;
import org.obm.push.backend.MSEvent;
import org.obm.push.backend.PIMDataType;
import org.obm.push.backend.obm22.impl.ObmSyncBackend;
import org.obm.push.data.calendarenum.AttendeeStatus;
import org.obm.push.exception.ActiveSyncException;
import org.obm.push.exception.CollectionNotFoundException;
import org.obm.push.exception.ObjectNotFoundException;
import org.obm.push.state.SyncState;
import org.obm.push.store.ISyncStorage;
import org.obm.push.store.InvitationStatus;
import org.obm.sync.auth.AccessToken;
import org.obm.sync.auth.AuthFault;
import org.obm.sync.auth.EventAlreadyExistException;
import org.obm.sync.auth.ServerFault;
import org.obm.sync.calendar.Attendee;
import org.obm.sync.calendar.CalendarInfo;
import org.obm.sync.calendar.Event;
import org.obm.sync.calendar.EventType;
import org.obm.sync.calendar.ParticipationState;
import org.obm.sync.client.calendar.AbstractEventSyncClient;
import org.obm.sync.items.EventChanges;

public class CalendarBackend extends ObmSyncBackend {

	private Map<PIMDataType, ObmSyncCalendarConverter> converters;

	public CalendarBackend(ISyncStorage storage) {
		super(storage);
		converters = new HashMap<PIMDataType, ObmSyncCalendarConverter>(2);
		converters.put(PIMDataType.CALENDAR, new EventConverter());
		converters.put(PIMDataType.TASKS, new TodoConverter());
	}

	public List<ItemChange> getHierarchyChanges(BackendSession bs) {
		List<ItemChange> ret = new LinkedList<ItemChange>();

		if (!bs.checkHint("hint.multipleCalendars", false)) {
			ItemChange ic = new ItemChange();
			String col = getDefaultCalendarName(bs);
			String serverId = "";
			try {
				Integer collectionId = getCollectionIdFor(bs.getDevId(), col);
				serverId = getServerIdFor(collectionId);
			} catch (ActiveSyncException e) {
				serverId = createCollectionMapping(bs.getDevId(), col);
				ic.setIsNew(true);
			}
			ic.setServerId(serverId);
			ic.setParentId("0");
			ic.setDisplayName(bs.getLoginAtDomain() + " calendar");
			ic.setItemType(FolderType.DEFAULT_CALENDAR_FOLDER);
			ret.add(ic);
			return ret;
		}
		// ADD EVENT FOLDER
		AbstractEventSyncClient cc = getCalendarClient(bs, null);
		AccessToken token = cc.login(bs.getLoginAtDomain(), bs.getPassword(),
				"o-push");
		try {
			CalendarInfo[] cals = cc.listCalendars(token);

			int idx = bs.getLoginAtDomain().indexOf("@");
			String domain = bs.getLoginAtDomain().substring(idx);

			for (CalendarInfo ci : cals) {
				ItemChange ic = new ItemChange();
				String col = "obm:\\\\" + bs.getLoginAtDomain()
						+ "\\calendar\\" + ci.getUid() + domain;
				Integer collectionId = getCollectionIdFor(bs.getDevId(), col);
				ic.setServerId(getServerIdFor(collectionId));
				ic.setParentId("0");
				ic.setDisplayName(ci.getMail() + " calendar");
				if (bs.getLoginAtDomain().equalsIgnoreCase(ci.getMail())) {
					ic.setItemType(FolderType.DEFAULT_CALENDAR_FOLDER);
				} else {
					ic.setItemType(FolderType.USER_CREATED_CALENDAR_FOLDER);
				}
				ret.add(ic);
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		cc.logout(token);

		return ret;
	}

	public List<ItemChange> getHierarchyTaskChanges(BackendSession bs) {
		List<ItemChange> ret = new ArrayList<ItemChange>(1);
		ItemChange ic = new ItemChange();
		String col = "obm:\\\\" + bs.getLoginAtDomain() + "\\tasks\\"
				+ bs.getLoginAtDomain();
		String serverId;
		try {
			Integer collectionId = getCollectionIdFor(bs.getDevId(), col);
			serverId = getServerIdFor(collectionId);
		} catch (ActiveSyncException e) {
			serverId = createCollectionMapping(bs.getDevId(), col);
			ic.setIsNew(true);
		}
		ic.setServerId(serverId);
		ic.setParentId("0");
		ic.setDisplayName(bs.getLoginAtDomain() + " tasks");
		ic.setItemType(FolderType.DEFAULT_TASKS_FOLDER);
		ret.add(ic);
		return ret;
	}

	public DataDelta getContentChanges(BackendSession bs, SyncState state,
			Integer collectionId) throws ActiveSyncException {
		List<ItemChange> addUpd = new LinkedList<ItemChange>();
		List<ItemChange> deletions = new LinkedList<ItemChange>();

		Date ls = state.getLastSync();
		AbstractEventSyncClient cc = getCalendarClient(bs, state.getDataType());
		AccessToken token = cc.login(bs.getLoginAtDomain(), bs.getPassword(),
				"o-push");
		String collectionPath = getCollectionPathFor(collectionId);
		String calendar = parseCalendarName(collectionPath);
		try {
			EventChanges changes = null;
			if (state.isLastSyncFiltred()) {
				changes = cc.getSyncEventDate(token, calendar, ls);
			} else {
				changes = cc.getSync(token, calendar, ls);
			}
			Event[] evs = changes.getUpdated();
			String userEmail = cc.getUserEmail(token);
			for (Event e : evs) {
				boolean canAdd = true;
				for (Attendee att : e.getAttendees()) {
					if (userEmail.equals(att.getEmail())
							&& ParticipationState.DECLINED.equals(att
									.getState())) {
						logger.info("Event["
								+ e.getDatabaseId()
								+ "] The participation state is declined. The event will be deleted on phone");
						canAdd = false;
						deletions.add(getDeletion(collectionId, e.getUid()));
						break;
					}
				}
				if (canAdd && e.getRecurrenceId() == null) {
					ItemChange change = getCalendarChange(bs,
							collectionId, e);
					addUpd.add(change);
				}
			}
			for (String del : changes.getRemoved()) {
				deletions.add(getDeletion(collectionId, del));
			}
			bs.addUpdatedSyncDate(collectionId, changes.getLastSync());
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}

		cc.logout(token);
		logger.info("getContentChanges(" + calendar + ", " + collectionPath
				+ ", lastSync: " + ls + ") => " + addUpd.size() + " entries.");

		DataDelta ret = new DataDelta(addUpd, deletions);
		filtreEvent(bs, state, collectionId, ret);
		return ret;
	}

	private void filtreEvent(BackendSession bs, SyncState state,
			Integer collectionId, DataDelta delta) {
		for (Iterator<ItemChange> it = delta.getChanges().iterator(); it
				.hasNext();) {
			ItemChange ic = it.next();
			if (ic.getData() instanceof MSEvent) {
				MSEvent event = (MSEvent) ic.getData();

				if (!storage.isMostRecentInvitation(collectionId,
						event.getUID(), event.getDtStamp())) {
					logger.info("A more recent event or email is synchronized on phone. The event["
							+ event.getUID() + "] will not synced");
					it.remove();
				} else {
					storage.markToDeletedSyncedInvitation(collectionId,
							event.getUID());
					Boolean update = storage.haveEmailToDeleted(collectionId,
							event.getUID());
					if (update) {
						storage.createOrUpdateInvitation(collectionId,
								event.getUID(), event.getDtStamp(),
								InvitationStatus.EVENT_TO_SYNCED, null);
						try {
							this.createOrUpdate(
									bs,
									collectionId,
									getServerIdFor(collectionId,
											event.getObmUID()), event);
						} catch (ActiveSyncException e) {
							logger.error(e.getMessage(), e);
						}
						it.remove();
					} else {
						storage.createOrUpdateInvitation(collectionId,
								event.getUID(), event.getDtStamp(),
								InvitationStatus.EVENT_SYNCED, state.getKey());
					}
				}
			}
		}

		try {
			List<String> eventUIDToDeleted = storage.getEventToDeleted(
					collectionId, state.getKey());
			
			List<String> eventToDeleted = getObmUIDFromUID(bs, eventUIDToDeleted);
			logger.info(eventToDeleted.size()
					+ " event(s) will be deleted on the PDA");
			List<ItemChange> its = this.getDeletions(collectionId,
					eventToDeleted);
			storage.updateInvitationStatus(InvitationStatus.DELETED,
					state.getKey(), collectionId,
					eventUIDToDeleted.toArray(new String[0]));
			delta.getDeletions().addAll(its);
		} catch (ActiveSyncException e) {
			logger.error(e.getMessage(), e);
		}
	}
	
	private List<String> getObmUIDFromUID(BackendSession bs, List<String> uids){
		List<String> ret = new ArrayList<String>(uids.size());
		AbstractEventSyncClient bc = getCalendarClient(bs,
				PIMDataType.CALENDAR);
		AccessToken token = bc.login(bs.getLoginAtDomain(),
				bs.getPassword(), "o-push");
		for(String uid : uids){
			try {
				Integer obmId = bc.getEventObmIdFromExtId(token, bs.getLoginAtDomain(), uid);
				if(obmId != null){
					ret.add(obmId.toString());
				} else {
					logger.error("Can't find obmId from UID["+uid+"]");
				}
			} catch (Exception e) {
				logger.error(e.getMessage(), e);
			}
		}
		bc.logout(token);
		return ret;
	}

	private String parseCalendarName(String collectionPath) {
		// parse obm:\\thomas@zz.com\calendar\sylvaing@zz.com
		int slash = collectionPath.lastIndexOf("\\");
		int at = collectionPath.lastIndexOf("@");

		return collectionPath.substring(slash + 1, at);
	}

	private ItemChange getCalendarChange(BackendSession bs, Integer collectionId,
			Event e) throws ActiveSyncException {
		ItemChange ic = new ItemChange();
		ic.setServerId(getServerIdFor(collectionId, e.getUid()));
		IApplicationData ev = convertEvent(bs, e);
		ic.setData(ev);
		return ic;
	}

	public String createOrUpdate(BackendSession bs, Integer collectionId,
			String serverId, IApplicationData data) throws ActiveSyncException {
		String collectionPath = getCollectionPathFor(collectionId);
		logger.info("createOrUpdate(" + bs.getLoginAtDomain() + ", "
				+ collectionPath + ", " + serverId + ")");
		AbstractEventSyncClient cc = getCalendarClient(bs, data.getType());
		AccessToken token = cc.login(bs.getLoginAtDomain(), bs.getPassword(),
		"o-push");
		try {
			String id = null;
			Event oldEvent = null;
			if (serverId != null) {
				int idx = serverId.lastIndexOf(":");
				id = serverId.substring(idx + 1);
				try {
					oldEvent = cc.getEventFromId(token, bs.getLoginAtDomain(), id);
				} catch (Exception e) {
					logger.error(e.getMessage(), e);
				}
			}

			String email = bs.getLoginAtDomain();
			try {
				email = cc.getUserEmail(token);
			} catch (Exception e) {
				logger.error("Error finding email: " + e.getMessage(), e);
			}
			Boolean isInternal = EventConverter.isInternalEvent(oldEvent, true);
			Event event = null;
			if(isInternal){
				event = converters.get(data.getType()).convertAsInternal(bs, oldEvent, data);
			} else {
				event = converters.get(data.getType()).convertAsExternal(bs, oldEvent, data);
			}

			Attendee att = new Attendee();
			att.setEmail(email);
			if (id != null) {
				try {
					event.setUid(id);
					cc.modifyEvent(token, parseCalendarName(collectionPath), event,
							true, true);
				} catch (Exception e) {
					logger.error(e.getMessage(), e);
				}
			} else {
				try {
					id = cc.createEvent(token, parseCalendarName(collectionPath),
							event, true);
				} catch (EventAlreadyExistException e) {
					id = getEventIdFromExtId(token, collectionPath, cc, event);
				} catch (Exception e) {
					logger.error(e.getMessage(), e);
				}
			}
			return getServerIdFor(collectionId, id);
		} finally {
			cc.logout(token);
		}
	}

	private String getEventIdFromExtId(AccessToken token, String collectionPath, AbstractEventSyncClient cc, Event event) {
		try {
			Integer obmid = cc.getEventObmIdFromExtId(token, parseCalendarName(collectionPath), event.getExtId());
			if(obmid != null){
				return obmid.toString();
			}
		} catch (ServerFault e) {
			logger.error(e.getMessage(), e);
		}
		return null;
	}

	public void delete(BackendSession bs, Integer collectionId, String serverId) throws ActiveSyncException {
		logger.info("delete serverId " + serverId);
		String collectionPath = getCollectionPathFor(collectionId);
		if (serverId != null) {
			int idx = serverId.indexOf(":");
			if (idx > 0) {
				String id = serverId.substring(idx + 1);
				AbstractEventSyncClient bc = getCalendarClient(bs,
						PIMDataType.CALENDAR);
				AccessToken token = bc.login(bs.getLoginAtDomain(),
						bs.getPassword(), "o-push");
				try {
					Event evr = bc.getEventFromId(token, bs.getLoginAtDomain(),
							id);
					if (evr != null) {
						if (bs.getLoginAtDomain().equals(evr.getOwnerEmail())) {
							bc.removeEvent(token,
									parseCalendarName(collectionPath), id, evr.getSequence(), true);
						} else {
							//using calendar backend, we retrieve MSEvent
							MSEvent mser = (MSEvent) convertEvent(bs, evr);
							updateUserStatus(bs, mser, AttendeeStatus.DECLINE, bc, token);
						}
					}
				} catch (Exception e) {
					logger.error(e.getMessage(), e);
				}
				bc.logout(token);
			}
		}
	}

	public String handleMeetingResponse(BackendSession bs, MSEvent event, AttendeeStatus status) {
		AbstractEventSyncClient calCli = getCalendarClient(bs, event.getType());
		AccessToken at = calCli.login(bs.getLoginAtDomain(), bs.getPassword(), "o-push");
		try {
			logger.info("handleMeetingResponse : " + event.getUID());
			Event obmEvent = createOrModifyInvitationEvent(bs, event, calCli, at);
			event.setObmUID(obmEvent.getUid());
			return updateUserStatus(bs, event, status, calCli, at);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		} finally {
			calCli.logout(at);
		}
		return null;
	}

	private Event createOrModifyInvitationEvent(BackendSession bs, MSEvent event,
			AbstractEventSyncClient calCli, AccessToken at) throws AuthFault,
			ServerFault {
		Event obmEvent = calCli.getEventFromExtId(at, bs.getLoginAtDomain(), event.getUID());
		Boolean isInternal = EventConverter.isInternalEvent(obmEvent, false);
		Event newEvent = null;
		if(isInternal){
			newEvent = new EventConverter().convertAsInternal(bs, event);
		} else {
			newEvent = new EventConverter().convertAsExternal(bs, event);
		}
		if (obmEvent == null) {
			logger.info("createOrModifyInvitationEvent : create new event " + event.getUID());
			String id = calCli.createEvent(at, bs.getLoginAtDomain(), newEvent, isInternal);
			return calCli.getEventFromId(at, bs.getLoginAtDomain(), id);
		} else {
			newEvent.setUid(obmEvent.getUid());
			if(!obmEvent.isInternalEvent()){
				logger.info("createOrModifyInvitationEvent : update event " + event.getUID());
				obmEvent = calCli.modifyEvent(at, bs.getLoginAtDomain(), newEvent, true, false);
			}
			return obmEvent;
		}
	}

	private String updateUserStatus(BackendSession bs, MSEvent msEvent,
			AttendeeStatus status, AbstractEventSyncClient calCli,
			AccessToken at) throws AuthFault, ServerFault,
			CollectionNotFoundException, ActiveSyncException {
		logger.info("update user status[" + status.toString()
					+ "] in calendar " + bs.getLoginAtDomain());
		ParticipationState participationStatus = EventConverter.status(null, status);
		calCli.changeParticipationState(at, bs.getLoginAtDomain(), msEvent.getUID(), participationStatus, msEvent.getObmSequence(), true);
		
		Integer collectionId = getCollectionIdFor(bs.getDevId(), getDefaultCalendarName(bs));
		return getServerIdFor(collectionId, msEvent.getObmUID());
	}

	public List<ItemChange> fetchItems(BackendSession bs,
			List<String> fetchServerIds) throws ObjectNotFoundException {
		List<ItemChange> ret = new LinkedList<ItemChange>();
		try {
			for (String serverId : fetchServerIds) {
				Integer id = getItemIdFor(serverId);
				if (id != null) {
					AbstractEventSyncClient calCli = getCalendarClient(bs,
							PIMDataType.CALENDAR);
					AccessToken token = calCli.login(bs.getLoginAtDomain(),
							bs.getPassword(), "o-push");

					Event e = calCli.getEventFromId(token,
							bs.getLoginAtDomain(), id.toString());
					ItemChange ic = new ItemChange();
					ic.setServerId(serverId);
					IApplicationData ev = convertEvent(bs, e);
					ic.setData(ev);
					ret.add(ic);
				}
			}
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
			throw new ObjectNotFoundException();
		}
		return ret;
	}

	private IApplicationData convertEvent(BackendSession bs, Event e) {
		if (EventType.VTODO.equals(e.getType())) {
			return converters.get(PIMDataType.TASKS).convert(bs, e);
		} else {
			return converters.get(PIMDataType.CALENDAR).convert(bs, e);
		}
	}
}
