package org.obm.push.impl;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.mortbay.jetty.HttpStatus;
import org.obm.push.backend.BackendSession;
import org.obm.push.backend.IBackend;
import org.obm.push.backend.IContinuation;
import org.obm.push.exception.ProcessingEmailException;
import org.obm.push.exception.SendEmailException;
import org.obm.push.exception.SmtpInvalidRcptException;
import org.obm.push.utils.FileUtils;

public abstract class MailRequestHandler implements IRequestHandler {

	protected Log logger = LogFactory.getLog(getClass());
	protected IBackend backend;

	protected MailRequestHandler(IBackend backend) {
		this.backend = backend;
	}

	@Override
	public void process(IContinuation continuation, BackendSession bs,
			ActiveSyncRequest request, Responder responder) throws IOException {
		InputStream mailContent = null;
		try {
			mailContent = new BufferedInputStream(request.getInputStream());
			mailContent.mark(mailContent.available());
		} catch (IOException e) {
			// 400 Bad Request
			responder.sendError(HttpStatus.ORDINAL_400_Bad_Request);
			return;
		}

		try {
			Boolean saveInSent = false;
			String sis = request.getParameter("SaveInSent");
			if (sis != null) {
				saveInSent = sis.equalsIgnoreCase("T");
			}
			if (logger.isDebugEnabled()) {
				logger.debug("Mail content:\n"
						+ new String(FileUtils.streamBytes(mailContent, false)));
				mailContent.reset();
			}
			this.process(continuation, bs, mailContent, saveInSent, request,
					responder);
			
		} catch (SmtpInvalidRcptException se) {
			notifyUser(bs, mailContent, se);
		} catch (ProcessingEmailException pe) {
			notifyUser(bs, mailContent, pe);
		} catch (SendEmailException e) {
			handleSendEmailException(e, responder, bs, mailContent);
		} catch (Throwable t) {
			notifyUser(bs, mailContent, t);
		}
	}

	private void handleSendEmailException(SendEmailException e,
			Responder responder, BackendSession bs, InputStream mailContent)
			throws IOException {
		if (e.getSmtpErrorCode() >= 500) {
			notifyUser(bs, mailContent, e);
		} else {
			logger.error(
					"Error while sending mail. HTTP error[500] will send to the pda.",
					e);
			responder.sendError(500);
		}
	}

	private void notifyUser(BackendSession bs, InputStream mailContent,
			Throwable t) {
		logger.error(
				"Error while sending mail. A mail with the error will be sent at the sender.",
				t);
		try {
			mailContent.reset();
		} catch (IOException e) {
		}
		backend.getErrorManager().sendMailHandlerError(bs, mailContent, t);
	}

	public abstract void process(IContinuation continuation, BackendSession bs,
			InputStream mailContent, Boolean saveInSent,
			ActiveSyncRequest request, Responder responder)
			throws SendEmailException, ProcessingEmailException, SmtpInvalidRcptException;

}
